#!/usr/bin/python

################################################################
#                                                              #
# Maligno - A Metasploit payload server                        #
# written by Juan J. Guelfo @ Encripto AS                      #
# post@encripto.no                                             #
#                                                              #
# Copyright 2013-2014 Encripto AS. All rights reserved.        #
#                                                              #
# Maligno is licensed under the FreeBSD license.               #
# http://www.freebsd.org/copyright/freebsd-license.html        #
#                                                              #
################################################################


import os, sys, ipcalc
import xml.etree.ElementTree as ET


def check_file_exists(filename):
    if not os.path.exists(filename):
        print "[-] File \"{0}\" cannot be found.\n".format(filename)
        sys.exit()


def get_metasploit_config(config_file):
    check_file_exists(config_file)        
    tree = ET.parse(config_file) 
    root = tree.getroot()
    
    config = []
    for child in root.find("Metasploit"):    
        if child.text and child.tag != "msfpayload":
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Metasploit>", config_file)
        sys.exit()   
    
    config = dict(config)
    try:
        if not config["path"] or not config["cachefolder"] or not config["resourcefolder"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Metasploit>", config_file)
        sys.exit()  
    
    return config


def get_payload_config(config_file):
    check_file_exists(config_file)        
    tree = ET.parse(config_file) 
    root = tree.getroot()
    
    tmp = []
    config = []
    msf = root.find("Metasploit")    
    for child in msf.findall("msfpayload"):    
        for c in child:
            tmp.append([c.tag, c.text])
           
        config.append(dict(tmp))
        
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<msfpayload>", config_file)
        sys.exit()   
    
    try:
        for p in config:          
            if not p["id"] or not p["payload"] or not p["lhost"] or not p["lport"] or not p["encoder"] or not p["badchars"] or not p["iterations"]:
                raise KeyError
        
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<msfpayload>", config_file)
        sys.exit()    
        
    return config


def get_scope_config(config_file):
    check_file_exists(config_file)        
    tree = ET.parse(config_file) 
    root = tree.getroot()
    
    scope = []
    for child in root.find("Scope"):
        ip = child.text
        if ip:
            if ip.find("/") > -1:
                for c in ipcalc.Network(ip):
                    scope.append(str(c))
            else:
                scope.append(ip)
   
    return scope   
    
    
def get_server_config(config_file):
    check_file_exists(config_file)        
    tree = ET.parse(config_file) 
    root = tree.getroot()
    
    config = []
    for child in root.find("Server"):    
        if child.text:
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Server>", config_file)
        sys.exit()   
    
    config = dict(config)
    
    try:
        if not config["addr"] or not config["port"] or not config["ssl"] or not config["sslcert"] or not config["proxy"] or not config["proxyport"] or not config["profile"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Server>", config_file)
        sys.exit()  
    
    return config   
    
    
def get_profile_request_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    config = []
    req = root.find("Server").find("Request")
    for child in req:    
        if child.text:
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Request>", profile_file)
        sys.exit()   
    
    config = dict(config)
    
    try:
        if not config["page"] or not config["parameter"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Request>", profile_file)
        sys.exit()  
    
    return config


def get_profile_response_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    config = []
    resp = root.find("Server").find("Response")
    for child in resp:    
        if child.text and child.tag != "header" and child.tag != "body":
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Response>", profile_file)
        sys.exit()   
    
    config = dict(config)
    
    try:
        if not config["protocol"] or not config["banner"] or not config["lastresort"] or not config["errortemplate"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Response>", profile_file)
        sys.exit()  
    
    return config


def get_profile_server_header_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    tmp = []
    config = []
    resp = root.find("Server").find("Response")
    for child in resp.findall("header"):    
        for c in child:
            tmp.append([c.tag, c.text])
           
        config.append(dict(tmp))
    
    return config    


def get_profile_network_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    config = []
    for child in root.find("Network"):    
        if child.text:
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Network>", profile_file)
        sys.exit()   
    
    config = dict(config)
    
    try:
        if not config["encoding"] or not config["rounds"] or not config["blocksize"] or not config["paddingchar"] or not config["secret"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Network>", profile_file)
        sys.exit()  
    
    return config    


def get_profile_client_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    config = []
    for child in root.find("Client"):    
        if child.text:
            config.append([child.tag, child.text])
    
    if not config:
        print "[-] No {0} sections have been found in {1}.".format("<Client>", profile_file)
        sys.exit()   
    
    config = dict(config)
    
    try:
        if not config["useragent"]:
            raise KeyError
    
    except KeyError:
        print "[-] There is an error in your {0} configuration in {1}.".format("<Client>", profile_file)
        sys.exit()  
    
    return config 


def get_profile_client_header_config(profile_file):
    check_file_exists(profile_file)        
    tree = ET.parse(profile_file) 
    root = tree.getroot()
    
    tmp = []
    config = []
    resp = root.find("Client")
    for child in resp.findall("header"):    
        for c in child:
            tmp.append([c.tag, c.text])
           
        config.append(dict(tmp))
    
    return config    
